const {
    isLittleEndian
} = require("nv-string-bytes")


function is_node() {
    return(
        typeof process === "object" &&
        process + "" === "[object process]" &&
        !process.versions.nw &&
        !(
            process.versions.electron &&
            process.type && 
            process.type !== "browser"
        )
    )
}

const btoa = (chars) => Buffer.from(chars, "binary").toString("base64");
const atob = (input) => Buffer.from(input, "base64").toString("binary");


function _createObjectURL(data, contentType = "") {
  const digits = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
  let buffer = `data:${contentType};base64,`;
  for (let i = 0, ii = data.length; i < ii; i += 3) {
    const b1 = data[i] & 0xff;
    const b2 = data[i + 1] & 0xff;
    const b3 = data[i + 2] & 0xff;
    const d1 = b1 >> 2,
          d2 = (b1 & 3) << 4 | b2 >> 4;
    const d3 = i + 1 < ii ? (b2 & 0xf) << 2 | b3 >> 6 : 64;
    const d4 = i + 2 < ii ? b3 & 0x3f : 64;
    buffer += digits[d1] + digits[d2] + digits[d3] + digits[d4];
  }
  return buffer;
}

function createObjectURL(data, contentType = "", forceDataSchema = false) {
  if (URL.createObjectURL && !forceDataSchema) {
    return URL.createObjectURL(new Blob([data], {
      type: contentType
    }));
  } else {
    return(_createObjectURL(data, contentType)) 
  }
}



function patch_global_this() {
    if(is_node()) {
        globalThis.btoa = btoa;
        globalThis.atob = atob;
        globalThis.ReadableStream
            = require("web-streams-polyfill/dist/ponyfill.js").ReadableStream;
        URL.createObjectURL = _createObjectURL;
    } else {
    }
}


const {
    shadow,
} = require("./capability")



const IsLittleEndianCached = {
  get value() {
    return shadow(this, "value", isLittleEndian());
  }
};


function isEvalSupported() {
  try {
    new Function(""); // eslint-disable-line no-new, no-new-func

    return true;
  } catch (e) {
    return false;
  }
}

const IsEvalSupportedCached = {
  get value() {
    return shadow(this, "value", isEvalSupported());
  }
};

module.exports = {
    is_node,btoa,atob,
    createObjectURL,
    patch_global_this,
    ////
    IsLittleEndianCached,
    isEvalSupported,
    IsEvalSupportedCached,
    ////
}
